from contextvars import ContextVar

from tesseral import AccessTokenClaims


class _AuthContext:
    access_token: str
    access_token_claims: AccessTokenClaims


_auth_context_var: ContextVar[_AuthContext] = ContextVar("auth_context_var")


def _extract_auth_context(name: str) -> _AuthContext:
    try:
        return _auth_context_var.get()
    except LookupError as e:
        raise RuntimeError(
            f"Called {name}() outside of an authenticated request. Did you forget to use RequireAuthMiddleware?"
        ) from e


def organization_id() -> str:
    return _extract_auth_context("organization_id").access_token_claims.organization.id  # type: ignore[union-attr,return-value]


def access_token_claims() -> AccessTokenClaims:
    return _extract_auth_context("access_token_claims").access_token_claims


def credentials() -> str:
    return _extract_auth_context("credentials").access_token



from .middleware import RequireAuthMiddleware
from .context import organization_id, access_token_claims, credentials

__all__ = [
    "RequireAuthMiddleware",
    "organization_id",
    "access_token_claims",
    "credentials",
]



from typing import Optional

from httpx import AsyncClient
from starlette.requests import Request
from starlette.responses import PlainTextResponse
from starlette.types import ASGIApp, Scope, Receive, Send
from tesseral.access_tokens import AsyncAccessTokenAuthenticator

from .context import _AuthContext, _auth_context_var

_PREFIX_BEARER = "Bearer "


class RequireAuthMiddleware:
    app: ASGIApp
    _access_token_authenticator: AsyncAccessTokenAuthenticator

    def __init__(
        self,
        app: ASGIApp,
        *,
        publishable_key: str,
        config_api_hostname: str = "config.tesseral.com",
        jwks_refresh_interval_seconds: int = 3600,
        http_client: Optional[AsyncClient] = None,
    ):
        self.app = app
        self._access_token_authenticator = AsyncAccessTokenAuthenticator(
            publishable_key=publishable_key,
            config_api_hostname=config_api_hostname,
            jwks_refresh_interval_seconds=jwks_refresh_interval_seconds,
            http_client=http_client,
        )

    async def __call__(self, scope: Scope, receive: Receive, send: Send):
        if scope["type"] != "http":
            await self.app(scope, receive, send)
            return

        request = Request(scope, receive)
        project_id = await self._access_token_authenticator.project_id()
        access_token = self._access_token(request, project_id)

        try:
            access_token_claims = (
                await self._access_token_authenticator.authenticate_access_token(
                    access_token=access_token
                )
            )

            auth_context = _AuthContext()
            auth_context.access_token = access_token
            auth_context.access_token_claims = access_token_claims
        except:  # noqa: E722
            response = PlainTextResponse("Unauthorized\n", status_code=401)
            await response(scope, receive, send)
            return

        token = _auth_context_var.set(auth_context)
        await self.app(scope, receive, send)
        _auth_context_var.reset(token)

    def _access_token(self, request: Request, project_id: str) -> str:
        auth_header = request.headers.get("authorization")
        if auth_header and auth_header.startswith(_PREFIX_BEARER):
            return auth_header[len(_PREFIX_BEARER) :]

        cookie_name = f"tesseral_{project_id}_access_token"
        if cookie_name in request.cookies:
            return request.cookies[cookie_name]

        return ""



from fastapi import FastAPI

from tesseral_fastapi import (
    RequireAuthMiddleware,
    credentials,
    access_token_claims,
    organization_id,
)

app = FastAPI()

app.add_middleware(
    RequireAuthMiddleware,
    publishable_key="publishable_key_7nvw48k6r4wazcpna9stb8tid",
    config_api_hostname="config.tesseral.com",
)


@app.get("/")
async def read_root():
    return {
        "credentials": credentials(),
        "access_token_claims": access_token_claims().organization.display_name,
        "organization_id": organization_id(),
    }
